package dao

import (
	"errors"
	dbModel "pump/model/db"
	"strings"

	"github.com/shopspring/decimal"
	"gorm.io/gorm"
	"gorm.io/gorm/clause"
	"gorm.io/gorm/logger"
)

// GetStorageHeight 获取上次缓存的高度
func (d *Dao) GetStorageHeight(key string) (value int, err error) {
	storage := new(dbModel.Height)
	err = d.db.Model(storage).Where("key = ?", key).First(storage).Error
	if errors.Is(err, gorm.ErrRecordNotFound) {
		// 返回一个默认的起始高度
		if strings.Contains(key, "l1") {
			return 6202096, nil
		}
		return 306880, nil
	}
	return storage.IntValue, err
}

// SetStorageHeight 设置上次缓存的高度
func (d *Dao) SetStorageHeight(key string, intValue int) (err error) {
	tx := d.db.Session(&gorm.Session{Logger: d.db.Logger.LogMode(logger.Error)})
	return tx.Model(&dbModel.Height{}).Clauses(clause.OnConflict{UpdateAll: true}).Create(&dbModel.Height{
		Key:      key,
		IntValue: intValue,
	}).Error
}

func (d *Dao) CreateToken(token *dbModel.Token) (err error) {
	return d.db.Clauses(clause.OnConflict{DoNothing: true}).Create(token).Error
}

func (d *Dao) UpdateToken(token string, newPrice, newMarketCap, newVolume, newVolume24H, newReserve0, newReserve1 decimal.Decimal) (err error) {
	token = strings.ToLower(token)
	return d.db.Model(&dbModel.Token{}).Where("token = ?", token).Updates(map[string]interface{}{
		"price":      newPrice,
		"market_cap": newMarketCap,
		"volume":     newVolume,
		"volume24h":  newVolume24H,
		"reserve0":   newReserve0,
		"reserve1":   newReserve1,
	}).Error
}

func (d *Dao) GraduateToken(token, txHash, uniV2Pair string) (err error) {
	token = strings.ToLower(token)
	uniV2Pair = strings.ToLower(uniV2Pair)
	return d.db.Model(&dbModel.Token{}).Where("token = ?", token).Updates(map[string]interface{}{
		"graduated":         true,
		"graduated_tx_hash": txHash,
		"uniswap_v2_pair":   uniV2Pair,
	}).Error
}

func (d *Dao) CreateTradeHistory(tradeHistory *dbModel.TradeHistory) (err error) {
	return d.db.Clauses(clause.OnConflict{DoNothing: true}).Create(tradeHistory).Error
}
